<?php
// $Id: dbi_mysqli.php,v 1.8 2002/02/11 15:09:27 erwin Exp $

// This class is based on the Perl DBI.
// New functionality should be added accordingly.
// Please refer to the DBI documentation (perldoc DBI).

/**
 * Class DBI
 */
class DBI extends STH
{
  /**
   * @var string
   */
  public $sDriverName = 'mySQL';
  /**
   * @var
   */
  public $sName;

  /**
   * @var mysqli
   */
  public $oSession;
  /**
   * @var bool
   */
  public $bDebug = false;

  /**
   * @var
   */
  public $err;
  /**
   * @var string
   */
  public $errstr = '';
  /**
   * @var bool
   */
  public $bRaiseError = false;

  /**
   * DBI constructor.
   * @param $db
   * @param $user
   * @param $password
   * @param $host
   * @param string $charset
   */
  function __construct($db, $user, $password, $host, $charset = 'utf8')
  {
    $this->oSession = @mysqli_connect($host, $user, $password, $db);
    if (@mysqli_connect_errno()) {
      echo 'Vehler bei der Verbindung zu MySQL: ' . mysqli_connect_error();
    }
    if (is_object($this->oSession) && !is_null($charset)) {
      if (!mysqli_set_charset($this->oSession, $charset)) {
        printf('Fehler beim Laden des Zeichensatz %s: %s' . "\n", $charset, mysqli_error($this->oSession));
        exit();
      }
    }
    $this->sName = $db;
  }

  /**
   * @todo mysql_list_tables to SHOW TABLES FROM
   * @return mixed
   */
  function list_tables()
  {
    /*$result=mysql_list_tables($this->sName);
    if (!$result) {
       echo "DB Fehler, Tabellen können nicht angezeigt werden\n";
       echo 'MySQL Fehler: ' . mysqli_error();
       exit;
    }
    return ($result);*/
  }

  /**
   * @param $query
   * @return $this
   */
  function prepare($query)
  {
    parent::__construct($this->oSession, $query, $this->bDebug);
    return $this;
  }

  /**
   * @param $query
   * @return int
   */
  function dbh_do($query)
  {
    $res = mysqli_query($query, $this->oSession);
    if ($res) {
      $this->err = 0;
      $this->errstr = '';
      return mysqli_affected_rows($this->oSession);
    } else {
      $this->err = mysqli_errno($this->oSession);
      $this->errstr = mysqli_error($this->oSession);
      return -1;
    }
  }

  /**
   * @return int
   */
  function autocommit()
  {
    echo '<br><b>Warning: Transactions not supported by this driver.</b><br>';
    return 0;
  }

  /**
   * @return int
   */
  function commit()
  {
    echo '<br><b>Warning: Transactions not supported by this driver.</b><br>';
    return 0;
  }

  /**
   * @return int
   */
  function rollback()
  {
    echo '<br><b>Warning: Transactions not supported by this driver.</b><br>';
    return 0;
  }

  /**
   * @param $query
   * @return string
   */
  function escape_string($query)
  {
    return mysqli_escape_string($this->oSession, $query);
  }

}

/**
 * Class STH
 */
class STH
{
  /**
   * @var
   */
  public $query;
  /**
   * @var
   */
  public $bDebug;
  /**
   * @var
   */
  public $oSession;
  /**
   * @var
   */
  public $res;
  /**
   * @var
   */
  public $row;
  /**
   * @var array
   */
  public $placeholders;
  /**
   * @var
   */
  public $dbi;

  /**
   * STH constructor.
   * @param $oSession
   * @param $query
   * @param $bDebug
   */
  function __construct($oSession, $query, $bDebug)
  {
    $this->oSession = $oSession;
    $this->query = $query;
    $this->bDebug = $bDebug;


    $this->placeholders = array();
    $quote = '';
    for ($i = 0; $i < strlen($query); ++$i) {
      if ($query[$i] == '\'') {
        if (empty($quote)) {
          $quote = '\'';
        } elseif ($quote == '\'') {
          $quote = '';
        }
      } elseif ($query[$i] == '"') {
        if (empty($quote)) {
          $quote = '"';
        } elseif ($quote == '"') {
          $quote = '';
        }
      } elseif ($query[$i] == '?') {
        if (empty($quote)) {
          array_push($this->placeholders, $i);
        }
      }
    }
  }

  /**
   * @return bool|mysqli_result
   */
  function execute()
  {

    $numargs = func_num_args();
    $arg_list = func_get_args();

    $parms = array();

    for ($i = 0; $i < $numargs; $i++) {
      if (is_array($arg_list[$i])) {
        while (list($dummy, $parm) = each($arg_list[$i])) {
          array_push($parms, $parm);
        }
      } else {
        array_push($parms, $arg_list[$i]);
      }
    }

    if (sizeof($parms) != sizeof($this->placeholders)) {
      print '<br><b>SQL Query contains ' . sizeof($this->placeholders) . ' placeholders but ' . sizeof($parms) . ' was passed</b><br>';
      exit;
    }

    if (sizeof($parms) > 0) {
      $query = substr($this->query, 0, $this->placeholders[0]);
      for ($i = 0; $i < sizeof($parms) - 1; ++$i) {
        $query .= $this->quote($parms[$i]) . substr($this->query, $this->placeholders[$i] + 1, $this->placeholders[$i + 1] - $this->placeholders[$i] - 1);
      }
      $query .= $this->quote($parms[$i]) . substr($this->query, $this->placeholders[$i] + 1);
    } else {
      $query = $this->query;
    }

    if ($this->bDebug) {
      $fd = @fopen('./tmp/dbi.' . $_SERVER['SERVER_NAME'] . '.log', 'a') or die ('Couldn\'t append to file');
      fputs($fd, $query . "\n==================\n");
      fclose($fd);
    }

    $this->res = @mysqli_query($this->oSession, $query);
    if (!$this->res) {
      $this->dbi=new stdClass();
      if (isset($this->dbi->bRaiseError) && $this->dbi->bRaiseError) {
        print '<br><b>Could not execute SQL query: " ' . $query . '"</b><br>';
        print '<br>' . mysqli_errno($this->oSession) . ' ' . mysqli_error($this->oSession) . '<br>';
        exit;
      }
      $this->dbi->errstr = mysqli_error($this->oSession);
      $this->dbi->err = mysqli_errno($this->oSession);
    }
    $this->row = 0;
    return $this->res;
  }

  /**
   * @return array|null
   */
  function fetchrow_array()
  {
    $this->row++;
    return mysqli_fetch_row($this->res);
  }

  /**
   * @return array|null
   */
  function fetchrow_hash()
  {
    $this->row++;
    return mysqli_fetch_array($this->res, MYSQLI_ASSOC);
  }

  /**
   *
   */
  function finish()
  {
    mysqli_free_result($this->res);
  }

  /**
   * @return int|string
   */
  function insert_id()
  {
    return mysqli_insert_id($this->oSession);
  }

  /**
   * @return int
   */
  function rows()
  {
    return mysqli_num_rows($this->res);
  }

  /**
   * @return int
   */
  function num_tablefields()
  {
    return mysqli_num_fields($this->res);
  }

  /**
   * @param $str
   * @return string
   */
  function quote($str)
  {
    if (get_magic_quotes_gpc()) {
      $str = stripslashes($str);
    }
    return '\'' . addslashes($str) . '\'';
  }
}